@@ -139,9 +139,8 @@ static void add_char_to_buf(sBuf* buf, char c)
     buf->mLen++;
 }
 
-void add_str_to_buf(sBuf* buf, char* str)
+void add_str_to_buf(sBuf* buf, char* str, int len)
 {
-    const int len = strlen(str);
     if(buf->mSize <= buf->mLen + len) {
         buf->mSize = (buf->mSize + len) * 1.8;
         buf->mBuf = REALLOC(buf->mBuf, buf->mSize);
@@ -458,10 +457,10 @@ void sCommand_delete(sCommand* self)
 
     FREE(self->mBlocks);
 
-    FREE(self->mRedirectsFileNames);
     for(i=0; i<self->mRedirectsNum; i++) {
         FREE(self->mRedirectsFileNames[i]);
     }
+    FREE(self->mRedirectsFileNames);
     FREE(self->mRedirects);
 }
 
@@ -816,14 +815,14 @@ static void sCommand_add_arg_to_command2(sCommand* self, MANAGED char* buf)
     self->mArgsRuntime[self->mArgsNumRuntime] = NULL;
 }
 
-static BOOL expand_env_to_command(ALLOC char** result, char* str, sCommand* command, sRunInfo* runinfo, sObject* nextin, sObject* fun);
+static BOOL expand_env_to_command(ALLOC char** result, char* str, sCommand* command, sRunInfo* runinfo, sObject* nextin);
 
-static BOOL expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunInfo* runinfo, sEnv* env, sObject* fun, sObject* nextin)
+static BOOL expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunInfo* runinfo, sEnv* env, sObject* nextin)
 {
     /// expand key ///
     char* key;
     if(env->mKeyEnv) {
-        if(!expand_env_to_command(ALLOC &key, env->mKey, command, runinfo, nextin, fun)) {
+        if(!expand_env_to_command(ALLOC &key, env->mKey, command, runinfo, nextin)) {
             return FALSE;
         }
     }
@@ -833,7 +832,7 @@ static BOOL expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunI
 
     switch(TYPE(object)) {
         case T_STRING: {
-            add_str_to_buf(buf, string_c_str(object));
+            add_str_to_buf(buf, string_c_str(object), string_length(object));
             }
             break;
 
@@ -842,8 +841,8 @@ static BOOL expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunI
                 int i;
                 for(i=0; i<vector_size(object); i++) {
                     sObject* object2 = vector_item(object, i);
-                    add_str_to_buf(buf, string_c_str(object2));
-                    if(i+1<vector_size(object)) add_str_to_buf(buf, " ");
+                    add_str_to_buf(buf, string_c_str(object2), string_length(object2));
+                    if(i+1<vector_size(object)) add_str_to_buf(buf, " ", 1);
                 }
             }
             else {
@@ -858,7 +857,7 @@ static BOOL expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunI
 
                 if(vector_size(object) > 0) {
                     object = vector_item(object, index);
-                    add_str_to_buf(buf, string_c_str(object));
+                    add_str_to_buf(buf, string_c_str(object), string_length(object));
                 }
             }
             break;
@@ -881,7 +880,7 @@ static BOOL expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunI
                     return FALSE;
                 }
 
-                add_str_to_buf(buf, string_c_str(object));
+                add_str_to_buf(buf, string_c_str(object), string_length(object));
             }
             break;
 
@@ -899,7 +898,7 @@ static BOOL expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunI
 }
 
 // if return is FALSE, no allocate result
-static BOOL expand_env_to_command(ALLOC char** result, char* str, sCommand* command, sRunInfo* runinfo, sObject* nextin, sObject* fun)
+static BOOL expand_env_to_command(ALLOC char** result, char* str, sCommand* command, sRunInfo* runinfo, sObject* nextin)
 {
     sBuf buf;
     memset(&buf, 0, sizeof(sBuf));
@@ -929,7 +928,7 @@ static BOOL expand_env_to_command(ALLOC char** result, char* str, sCommand* comm
                     char* env2 = getenv(env->mName);
 
                     if(env2) {
-                        add_str_to_buf(&buf, env2);
+                        add_str_to_buf(&buf, env2, strlen(env2));
                     }
                     else {
                         FREE(buf.mBuf);
@@ -940,7 +939,7 @@ static BOOL expand_env_to_command(ALLOC char** result, char* str, sCommand* comm
                     }
                 }
                 else {
-                    if(!expand_variable(command, object, &buf, runinfo, env, fun,nextin)) {
+                    if(!expand_variable(command, object, &buf, runinfo, env,nextin)) {
                         FREE(buf.mBuf);
                         return FALSE;
                     }
@@ -959,7 +958,7 @@ static BOOL expand_env_to_command(ALLOC char** result, char* str, sCommand* comm
                     return FALSE;
                 }
 
-                add_str_to_buf(&buf, SFD(nextout).fdbuf.mBuf);
+                add_str_to_buf(&buf, SFD(nextout).fdbuf.mBuf, SFD(nextout).fdbuf.mBufLen);
             }
         }
         else {
@@ -986,7 +985,7 @@ static BOOL get_option(MANAGED char* arg, sCommand* command, sObject* fun, sObje
 
         if(command->mArgsEnv[*i+1]) {
             char* expanded_str;
-            if(!expand_env_to_command(ALLOC &expanded_str, command->mArgs[*i+1], command, runinfo, nextin, fun)) {
+            if(!expand_env_to_command(ALLOC &expanded_str, command->mArgs[*i+1], command, runinfo, nextin)) {
                 FREE(arg);
                 return FALSE;
             }
@@ -1060,7 +1059,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sObje
         /// variable ///
         else if(command->mArgsEnv[i]) {
             char* expanded_str;
-            if(!expand_env_to_command(ALLOC &expanded_str, command->mArgs[i], command, runinfo, nextin, fun)) {
+            if(!expand_env_to_command(ALLOC &expanded_str, command->mArgs[i], command, runinfo, nextin)) {
                 return FALSE;
             }
 
@@ -1075,6 +1074,24 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sObje
     return TRUE;
 }
 
+BOOL sCommand_expand_env_redirect(sCommand* command, sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    int i;
+    for(i=0; i<command->mRedirectsNum; i++) {
+        char* fname = command->mRedirectsFileNames[i];
+        if(fname[0] == PARSER_MAGIC_NUMBER_ENV) {
+            char* expanded_str;
+            if(!expand_env_to_command(ALLOC &expanded_str, fname, command, runinfo, nextin)) {
+                return FALSE;
+            }
+            FREE(command->mRedirectsFileNames[i]);
+            command->mRedirectsFileNames[i] = expanded_str;
+        }
+    }
+
+    return TRUE;
+}
+
 int block_gc_children_mark(sObject* self)
 {
     int count = 0;
