@@ -47,7 +47,7 @@ BOOL cmd_each(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         char* argument;
         if(argument = sCommand_option_with_argument_item(command, "-number")) {
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -65,7 +65,8 @@ BOOL cmd_each(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 int j;
                 for(j=0; j<number && i+j<vector_size(SFD(nextin).fdbuf.mLines); j++)
                 {
-                    if(!fd_write(nextin2, vector_item(SFD(nextin).fdbuf.mLines, i+j))) {
+                    char* str = vector_item(SFD(nextin).fdbuf.mLines, i+j);
+                    if(!fd_write(nextin2, str, strlen(str))) {
                         err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -88,7 +89,7 @@ BOOL cmd_each(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
         }
         else {
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -101,7 +102,9 @@ BOOL cmd_each(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             for(i=0; i<vector_size(SFD(nextin).fdbuf.mLines); i++) {
                 fd_clear(nextin2);
 
-                if(!fd_write(nextin2, vector_item(SFD(nextin).fdbuf.mLines, i))) {
+                char* str = vector_item(SFD(nextin).fdbuf.mLines, i);
+
+                if(!fd_write(nextin2, str, strlen(str))) {
                     err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
@@ -161,12 +164,12 @@ BOOL cmd_join(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             for(i=0; i<vector_size(SFD(nextin).fdbuf.mLines)-1; i++) {
                 sObject* line = STRING_NEW_STACK(vector_item(SFD(nextin).fdbuf.mLines, i));
                 string_chomp(line);
-                if(!fd_write(nextout, string_c_str(line))) {
+                if(!fd_write(nextout, string_c_str(line), string_length(line))) {
                     err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
-                if(!fd_write(nextout, field)) {
+                if(!fd_write(nextout, field, strlen(field))) {
                     err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
@@ -175,20 +178,20 @@ BOOL cmd_join(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             sObject* line = STRING_NEW_STACK(vector_item(SFD(nextin).fdbuf.mLines, i));
             string_chomp(line);
-            if(!fd_write(nextout, string_c_str(line))) {
+            if(!fd_write(nextout, string_c_str(line), string_length(line))) {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
 
-            if(!fd_write(nextout, "\n")) {
+            if(!fd_write(nextout, "\n", 1)) {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
         }
 
-        if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+        if(SFD(nextin).fdbuf.mBufLen == 0) {
             runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
         }
         else {
@@ -218,7 +221,7 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
 
     if(command->mArgsNumRuntime > 1 && command->mBlocksNum == command->mArgsNumRuntime-1 && runinfo->mFilter) {
-        if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+        if(SFD(nextin).fdbuf.mBufLen == 0) {
             runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
         }
         else {
@@ -276,7 +279,8 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         for(j=first; j<=second; j++) {
                             fd_clear(nextin2);
 
-                            if(!fd_write(nextin2, vector_item(SFD(nextin).fdbuf.mLines, j))) {
+                            char* str = vector_item(SFD(nextin).fdbuf.mLines, j);
+                            if(!fd_write(nextin2,str,strlen(str))) {
                                 err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
@@ -298,7 +302,8 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         for(j=first; j>=second; j--) {
                             fd_clear(nextin2);
 
-                            if(!fd_write(nextin2, vector_item(SFD(nextin).fdbuf.mLines, j))) {
+                            char* str = vector_item(SFD(nextin).fdbuf.mLines, j);
+                            if(!fd_write(nextin2, str, strlen(str))) {
                                 err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
@@ -328,7 +333,8 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                     sObject* nextin2 = FD_NEW_STACK(kFDKindBuf, 0);
 
-                    if(!fd_write(nextin2, vector_item(SFD(nextin).fdbuf.mLines, num))) {
+                    char* str = vector_item(SFD(nextin).fdbuf.mLines, num);
+                    if(!fd_write(nextin2, str, strlen(str))){
                         err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -359,10 +365,10 @@ static int sort_fun(void* left, void* right, sRunInfo* runinfo, sObject* nextin2
     char* right2 = right;
 
     fd_clear(nextin2);
-    if(!fd_write(nextin2, left2)) {
+    if(!fd_write(nextin2, left2, strlen(left2))) {
         return -1;
     }
-    if(!fd_write(nextin2, right2)) {
+    if(!fd_write(nextin2, right2, strlen(right2))) {
         return -1;
     }
 
@@ -543,7 +549,7 @@ BOOL cmd_sort(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                 for(i=0; i<vector_size(SFD(nextin).fdbuf.mLines); i++) {
                     char* line = vector_item(SFD(nextin).fdbuf.mLines, i);
-                    if(!fd_write(nextout, line)) {
+                    if(!fd_write(nextout, line, strlen(line))) {
                         err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -551,7 +557,7 @@ BOOL cmd_sort(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
 
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -570,14 +576,14 @@ BOOL cmd_sort(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 int i;
                 for(i=0; i<vector_size(SFD(nextin).fdbuf.mLines); i++) {
                     char* line = vector_item(SFD(nextin).fdbuf.mLines, i);
-                    if(!fd_write(nextout, line)) {
+                    if(!fd_write(nextout, line, strlen(line))) {
                         err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
                 }
             }
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
