@@ -32,14 +32,14 @@ BOOL cmd_jobs(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     int i;
     for(i=0; i<vector_size(gJobs); i++) {
         sObject* job = vector_item(gJobs, i);
-        snprintf(buf, BUFSIZ, "[%d] name: %s pid&pgrp: %d", i+1, SJOB(job).mName, SJOB(job).mPGroup);
-        if(!fd_write(nextout, buf)) {
+        int size = snprintf(buf, BUFSIZ, "[%d] name: %s pid&pgrp: %d", i+1, SJOB(job).mName, SJOB(job).mPGroup);
+        if(!fd_write(nextout, buf, size)) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
 
-        if(!fd_write(nextout, "\n")) {
+        if(!fd_write(nextout, "\n", 1)) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
@@ -237,9 +237,9 @@ BOOL cmd_stackinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     char buf[BUFSIZ];
 
-    snprintf(buf, BUFSIZ, "slot size %d\npool size %d\nall object number %d\n", stack_slot_size(), stack_page_size(), stack_slot_size()*stack_page_size());
+    int size = snprintf(buf, BUFSIZ, "slot size %d\npool size %d\nall object number %d\n", stack_slot_size(), stack_page_size(), stack_slot_size()*stack_page_size());
 
-    if(!fd_write(nextout, buf)) {
+    if(!fd_write(nextout, buf, size)) {
         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         return FALSE;
@@ -262,9 +262,9 @@ BOOL cmd_stackframe(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         char*obj_kind[16] = {
             NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "external object"
         };
-        snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[TYPE(object2)]);
+        int size = snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[TYPE(object2)]);
 
-        if(!fd_write(nextout, buf)) {
+        if(!fd_write(nextout, buf, size)) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
@@ -324,9 +324,9 @@ BOOL cmd_funinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         }
         strcat(command_name, command->mArgs[0]);
 
-        snprintf(buf, BUFSIZ, "source name: %s\nsource line: %d\nrun nest level: %d\ncurrent object: %s\nreciever object: %s\ncommand name: %s\n", runinfo2->mSName, runinfo2->mSLine, runinfo2->mNestLevel, current_object, reciever_object, command_name);
+        int size = snprintf(buf, BUFSIZ, "source name: %s\nsource line: %d\nrun nest level: %d\ncurrent object: %s\nreciever object: %s\ncommand name: %s\n", runinfo2->mSName, runinfo2->mSLine, runinfo2->mNestLevel, current_object, reciever_object, command_name);
 
-        if(!fd_write(nextout, buf)) {
+        if(!fd_write(nextout, buf, size)) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
@@ -343,9 +343,9 @@ BOOL cmd_gcinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     sCommand* command = runinfo->mCommand;
     char buf[BUFSIZ];
 
-    snprintf(buf, BUFSIZ, "free objects %d\nused objects %d\nall object number %d\nslot size %d\npool size %d\n", gc_free_objects_num(), gc_used_objects_num(), gc_slot_size()*gc_pool_size(), gc_slot_size(), gc_pool_size());
+    int size = snprintf(buf, BUFSIZ, "free objects %d\nused objects %d\nall object number %d\nslot size %d\npool size %d\n", gc_free_objects_num(), gc_used_objects_num(), gc_slot_size()*gc_pool_size(), gc_slot_size(), gc_pool_size());
 
-    if(!fd_write(nextout, buf)) {
+    if(!fd_write(nextout, buf, size)) {
         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         return FALSE;
@@ -379,9 +379,9 @@ BOOL cmd_sweep(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int num = gc_sweep();
 
         char buf[BUFSIZ];
-        snprintf(buf, BUFSIZ, "%d objects deleted\n", num);
+        int size = snprintf(buf, BUFSIZ, "%d objects deleted\n", num);
 
-        if(!fd_write(nextout, buf)) {
+        if(!fd_write(nextout, buf, size)) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
@@ -424,18 +424,18 @@ BOOL cmd_print(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     if(command->mArgsNumRuntime > 1) {
         int i;
         for(i=1; i<command->mArgsNumRuntime-1; i++) {
-            if(!fd_write(out, command->mArgsRuntime[i])) {
+            if(!fd_write(out, command->mArgsRuntime[i], strlen(command->mArgsRuntime[i]))) {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
-            if(!fd_write(out, " ")) {
+            if(!fd_write(out, " ", 1)) {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
         }
-        if(!fd_write(out, command->mArgsRuntime[i])) {
+        if(!fd_write(out, command->mArgsRuntime[i], strlen(command->mArgsRuntime[i]))) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
@@ -444,7 +444,7 @@ BOOL cmd_print(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
     else if(runinfo->mFilter) {
         if(sCommand_option_item(command, "-read-from-error")) {
-            if(!fd_write(out, SFD(gStderr).fdbuf.mBuf)) {
+            if(!fd_write(out, SFD(gStderr).fdbuf.mBuf, SFD(gStderr).fdbuf.mBufLen)) {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
@@ -452,14 +452,14 @@ BOOL cmd_print(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             fd_clear(gStderr);
         }
         else {
-            if(!fd_write(out, SFD(nextin).fdbuf.mBuf)) {
+            if(!fd_write(out, SFD(nextin).fdbuf.mBuf, SFD(nextin).fdbuf.mBufLen)) {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
         }
 
-        if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+        if(SFD(nextin).fdbuf.mBufLen == 0) {
             runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
         }
         else {
@@ -531,6 +531,7 @@ BOOL cmd_eval(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int sline = 1;
         char buf2[BUFSIZ];
         snprintf(buf2, BUFSIZ, "%s %d: eval", runinfo->mSName, runinfo->mSLine);
+
         if(parse(buf, buf2, &sline, block, NULL)) {
             if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
                 err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -547,7 +548,7 @@ BOOL cmd_eval(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         stack_end_stack();
 
-        if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+        if(SFD(nextin).fdbuf.mBufLen == 0) {
             runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
         }
         else {
@@ -569,9 +570,9 @@ BOOL cmd_msleep(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int i;
         for(i=0; i < n*5; i++) {
             char buf[32];
-            snprintf(buf, 32, "%c\033[D", c[i % 3]);
+            int size = snprintf(buf, 32, "%c\033[D", c[i % 3]);
 
-            if(!fd_write(nextout, buf)) {
+            if(!fd_write(nextout, buf, size)) {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
@@ -629,9 +630,9 @@ BOOL cmd_time(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int sec2 = sec - minuts * 60;
 
         char buf[1024];
-        snprintf(buf, 1024, "%d sec(%d minuts %d sec)\n", sec, minuts, sec2);
+        int size = snprintf(buf, 1024, "%d sec(%d minuts %d sec)\n", sec, minuts, sec2);
 
-        if(!fd_write(nextout, buf) ) {
+        if(!fd_write(nextout, buf, size)) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
@@ -779,7 +780,7 @@ BOOL cmd_try(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 BOOL cmd_errmsg(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
-    if(!fd_write(nextout, string_c_str(gErrMsg))) {
+    if(!fd_write(nextout, string_c_str(gErrMsg), string_length(gErrMsg))) {
         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         return FALSE;
@@ -811,9 +812,9 @@ BOOL cmd_block(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     if(sCommand_option_item(command, "-number")) {
         if(SFUN(fun).mArgBlocks) {
             char buf[32];
-            snprintf(buf, 32, "%d\n", vector_size(SFUN(fun).mArgBlocks));
+            int size = snprintf(buf, 32, "%d\n", vector_size(SFUN(fun).mArgBlocks));
 
-            if(!fd_write(nextout, buf))
+            if(!fd_write(nextout, buf, size))
             {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
@@ -855,7 +856,7 @@ BOOL cmd_block(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         if(SFUN(fun).mArgBlocks && block_num < vector_size(SFUN(fun).mArgBlocks)) {
             sObject* block = vector_item(SFUN(fun).mArgBlocks, block_num);
 
-            if(!fd_write(nextout, SBLOCK(block).mSource))
+            if(!fd_write(nextout, SBLOCK(block).mSource, strlen(SBLOCK(block).mSource)))
             {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
