@@ -31,7 +31,7 @@ BOOL cmd_condition_n(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     if(runinfo->mFilter) {
         runinfo->mRCode = RCODE_NFUN_FALSE;
-        if(strcmp(SFD(nextin).fdbuf.mBuf, "") != 0) {
+        if(SFD(nextin).fdbuf.mBufLen > 0) {
             runinfo->mRCode = 0;
         }
     }
@@ -45,7 +45,7 @@ BOOL cmd_condition_z(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     if(runinfo->mFilter) {
         runinfo->mRCode = RCODE_NFUN_FALSE;
-        if(strcmp(SFD(nextin).fdbuf.mBuf, "") == 0) {
+        if(SFD(nextin).fdbuf.mBufLen == 0) {
             runinfo->mRCode = 0;
         }
     }
@@ -729,8 +729,8 @@ BOOL cmd_condition_re(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     int point = str_pointer2kanjipos(code, target, target + r2);
 
                     char buf[1024];
-                    snprintf(buf, 1024, "%d\n", point);
-                    if(!fd_write(nextout, buf)) {
+                    int size = snprintf(buf, 1024, "%d\n", point);
+                    if(!fd_write(nextout, buf, size)) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         onig_region_free(region, 1);
@@ -745,8 +745,8 @@ BOOL cmd_condition_re(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         int point = str_pointer2kanjipos(code, target, target + region->beg[i]);
 
                         char buf[1024];
-                        snprintf(buf, 1024, "%d\n", point);
-                        if(!fd_write(nextout, buf)) {
+                        int size = snprintf(buf, 1024, "%d\n", point);
+                        if(!fd_write(nextout, buf, size)) {
                             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                             onig_region_free(region, 1);
@@ -756,8 +756,8 @@ BOOL cmd_condition_re(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                         point = str_pointer2kanjipos(code, target, target + region->end[i]);
 
-                        snprintf(buf, 1024, "%d\n", point);
-                        if(!fd_write(nextout, buf)) {
+                        size = snprintf(buf, 1024, "%d\n", point);
+                        if(!fd_write(nextout, buf, size)) {
                             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                             onig_region_free(region, 1);
