@@ -74,6 +74,10 @@ static void object_delete(sObject* obj)
     case T_EXTOBJ:
         SEXTOBJ(obj).mFreeFun(SEXTOBJ(obj).mObject);
         break;
+
+    case T_EXTERNAL:
+        external_delete_gc(obj);
+        break;
     }
 }
 
