@@ -64,12 +64,11 @@ sObject* fd_new_from_stack(int kind, int fd)
     }
     else {
         SFD(self).fdbuf.mBuf = MALLOC(1024);
+        SFD(self).fdbuf.mBuf[0] = 0;
 
         SFD(self).fdbuf.mBufSize = 1024;
         SFD(self).fdbuf.mBufLen = 0;
 
-        *SFD(self).fdbuf.mBuf = 0;
-
         SFD(self).fdbuf.mLines = VECTOR_NEW_MALLOC(32);
     }
 
@@ -92,14 +91,13 @@ void fd_delete_stack(sObject* self)
 
 // TRUE: Success
 // FALSE: signal interrupt. set RCODE_SIGNAL_INTERRUPT on rcode and err message
-BOOL bufsiz_write(int fd, char* buf)
+BOOL bufsiz_write(int fd, char* buf, int buf_size)
 {
-    int len = strlen(buf);
     char* p = buf;
-    while(*p) {
+    while(p - buf < buf_size) {
         int size;
-        if(len - (p - buf) < BUFSIZ) {
-            size = len - (p-buf);
+        if(buf_size - (p - buf) < BUFSIZ) {
+            size = buf_size - (p-buf);
         }
         else {
             size = BUFSIZ;
@@ -161,7 +159,7 @@ void fd_split(sObject* self, enum eLineField lf)
                     vector_add(v, line);
                     new_line = p;
                 }
-                else if(*p == 0) {
+                else if(p - SFD(self).fdbuf.mBuf >= SFD(self).fdbuf.mBufLen) {
                     const int size = p - new_line;
                     if(size > 0) {
                         char* line = MALLOC(size + 1 + 2);
@@ -200,7 +198,7 @@ void fd_split(sObject* self, enum eLineField lf)
                     p++;
                     new_line = p;
                 }
-                else if(*p == 0) {
+                else if(p - SFD(self).fdbuf.mBuf >= SFD(self).fdbuf.mBufLen) {
                     const int size = p - new_line;
                     if(size > 0) {
                         char* line = MALLOC(size + 1 + 1);
@@ -221,7 +219,6 @@ void fd_split(sObject* self, enum eLineField lf)
     }
 }
 
-
 // TRUE: Success
 // FALSE: signal interrupt
 static BOOL memcpy_buf(char* dest, char* src, size_t size)
@@ -265,52 +262,23 @@ void fd_clear(sObject* self)
 
 // TRUE: Success
 // FALSE: signal interrupt. set RCODE_SIGNAL_INTERRUPT on rcode and err message
-BOOL fd_write(sObject* self, char* str)
+BOOL fd_write(sObject* self, char* str, int len)
 {
     if(SFD(self).mKind == kFDKindFD) {
-        if(write(SFD(self).fd.mFD, str, strlen(str)) < 0) {
+        if(write(SFD(self).fd.mFD, str, len) < 0) {
             return FALSE;
         }
     }
     else {
-        const int len = strlen(str);
-
-        if((SFD(self).fdbuf.mBufLen+len) >= SFD(self).fdbuf.mBufSize) {
-            int new_size = (SFD(self).fdbuf.mBufSize + len) * 1.8;
+        if((SFD(self).fdbuf.mBufLen+len+1) >= SFD(self).fdbuf.mBufSize) {
+            int new_size = (SFD(self).fdbuf.mBufSize + len+1) * 1.8;
 
             SFD(self).fdbuf.mBuf = REALLOC(SFD(self).fdbuf.mBuf, new_size);
-
             SFD(self).fdbuf.mBufSize = new_size;
         }
 
-        if(!memcpy_buf(SFD(self).fdbuf.mBuf + SFD(self).fdbuf.mBufLen, str, len + 1)) {
-            return FALSE;
-        }
-        SFD(self).fdbuf.mBufLen += len;
-    }
 
-    return TRUE;
-}
-
-BOOL fd_write3(sObject* self, char* str, int size)
-{
-    if(SFD(self).mKind == kFDKindFD) {
-        if(write(SFD(self).fd.mFD, str, size) < 0) {
-            return FALSE;
-        }
-    }
-    else {
-        const int len = size;
-
-        if((SFD(self).fdbuf.mBufLen+len) >= SFD(self).fdbuf.mBufSize) {
-            int new_size = (SFD(self).fdbuf.mBufSize + len) * 1.8;
-
-            SFD(self).fdbuf.mBuf = REALLOC(SFD(self).fdbuf.mBuf, new_size);
-
-            SFD(self).fdbuf.mBufSize = new_size;
-        }
-
-        if(!memcpy_buf(SFD(self).fdbuf.mBuf + SFD(self).fdbuf.mBufLen, str, size)) {
+        if(!memcpy_buf(SFD(self).fdbuf.mBuf + SFD(self).fdbuf.mBufLen, str, len)) {
             return FALSE;
         }
         SFD(self).fdbuf.mBufLen += len;
@@ -330,8 +298,8 @@ BOOL fd_writec(sObject* self, char c)
         }
     }
     else {
-        if((SFD(self).fdbuf.mBufLen+1) >= SFD(self).fdbuf.mBufSize) {
-            int new_size = (SFD(self).fdbuf.mBufSize + 1) * 1.8;
+        if((SFD(self).fdbuf.mBufLen) >= SFD(self).fdbuf.mBufSize) {
+            int new_size = (SFD(self).fdbuf.mBufSize) * 1.8;
             SFD(self).fdbuf.mBuf = REALLOC(SFD(self).fdbuf.mBuf, new_size);
             SFD(self).fdbuf.mBufSize = new_size;
         }
@@ -353,12 +321,12 @@ BOOL fd_writec(sObject* self, char c)
 BOOL fd_flash(sObject* self, int fd)
 {
     if(SFD(self).mKind == kFDKindBuf) {
-        if(!bufsiz_write(fd, SFD(self).fdbuf.mBuf)) {
+        if(!bufsiz_write(fd, SFD(self).fdbuf.mBuf, SFD(self).fdbuf.mBufLen)) {
             return FALSE;
         }
 
-        SFD(self).fdbuf.mBuf[0] = 0;
         SFD(self).fdbuf.mBufLen = 0;
+        SFD(self).fdbuf.mBuf[0] = 0;
     }
 
     return TRUE;
@@ -384,43 +352,34 @@ void job_delete_gc(sObject* self)
 
 static BOOL nextout_reader(sObject* nextout, int* pipeoutfds, sRunInfo* runinfo, char* program)
 {
-    //if(!gxyzshSigUser) {
-        char buf[BUFSIZ+1];
-        while(1) {
-            if(gxyzshSigInt) {
-                err_msg("signal interrupt1", runinfo->mSName, runinfo->mSLine, program);
-                gxyzshSigInt = FALSE;
-                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                return FALSE;
-            }
-            int r = read(pipeoutfds[0], buf, BUFSIZ);
-            if(r < 0) {
-                if(errno == EINTR) {
-                    if(!fd_write(nextout, buf)) {
-                        err_msg("signal interrupt2", runinfo->mSName, runinfo->mSLine, program);
-                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                        return FALSE;
-                    }
-                    break;
-                }
-                else {
-                    perror("read mpipe");
-                    exit(1);
-                }
-            }
-            buf[r] = 0;
-
-            if(!fd_write(nextout, buf)) {
-                err_msg("signal interrupt3", runinfo->mSName, runinfo->mSLine, program);
-                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                return FALSE;
-            }
-
-            if(r == 0) {
+    char buf[BUFSIZ+1];
+    while(1) {
+        if(gxyzshSigInt) {
+            err_msg("signal interrupt1", runinfo->mSName, runinfo->mSLine, program);
+            gxyzshSigInt = FALSE;
+            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+            return FALSE;
+        }
+        int size = read(pipeoutfds[0], buf, BUFSIZ);
+        if(size < 0) {
+            if(errno == EINTR) {
                 break;
             }
+            else {
+                perror("read pipe");
+                exit(1);
+            }
         }
-    //}
+        else if(size == 0) {
+            break;
+        }
+
+        if(!fd_write(nextout, buf, size)) {
+            err_msg("signal interrupt3", runinfo->mSName, runinfo->mSLine, program);
+            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+            return FALSE;
+        }
+    }
 
     (void)close(pipeoutfds[0]);
 
@@ -452,14 +411,12 @@ static void nextin_writer(pid_t pid, sObject* nextin, int* pipeinfds, int* pipeo
         }
         (void)close(pipeinfds[0]);
 
-        //if(!gxyzshSigUser) {
-            if(!bufsiz_write(pipeinfds[1], SFD(nextin).fdbuf.mBuf)) {
-                if(errno != EPIPE ) {
-                    perror("write memory pipe");
-                    exit(1);
-                }
+        if(!bufsiz_write(pipeinfds[1], SFD(nextin).fdbuf.mBuf, SFD(nextin).fdbuf.mBufLen)) {
+            if(errno != EPIPE ) {
+                perror("write memory pipe");
+                exit(1);
             }
-        //}
+        }
         (void)close(pipeinfds[1]);
 
         exit(0);
@@ -715,6 +672,9 @@ static BOOL run_external_command(char* program, sObject* nextin, sObject* nextou
     if(nextin == gStdin && SFD(gStdin).fdbuf.mBufLen == 0) {
         nextin2 = 0;
     }
+    else if(SFD(nextin).mKind == kFDKindFD) {
+        nextin2 = SFD(nextin).fd.mFD;
+    }
     else {
         if(pipe(pipeinfds) < 0) {
             perror("pipe");
@@ -726,6 +686,15 @@ static BOOL run_external_command(char* program, sObject* nextin, sObject* nextou
         nextout2 = 1;
         nexterr2 = 2;
     }
+    else if(SFD(nextout).mKind == kFDKindFD) {
+        nextout2 = SFD(nextout).fd.mFD;
+
+        if(pipe(pipeerrfds) < 0) {
+            perror("pipe");
+            exit(1);
+        }
+        nexterr2 = pipeerrfds[1];
+    }
     else {
         if(pipe(pipeoutfds) < 0) {
             perror("pipe");
@@ -749,16 +718,14 @@ static BOOL run_external_command(char* program, sObject* nextin, sObject* nextou
 
     /// a child process ///
     if(pid == 0) {
-        if(SFD(nextin).mKind == kFDKindFD) { (void)close(SFD(nextin).fd.mFD); }
-        if(SFD(nextout).mKind == kFDKindFD) { (void)close(SFD(nextout).fd.mFD); }
-
         if(nextin2 != 0) { (void)close(pipeinfds[1]); }
         if(nextout2 != 1) { (void)close(pipeoutfds[0]); }
         if(nexterr2 != 2) {(void)close(pipeerrfds[0]); }
 
         pid = getpid();
 
-        if(nextout2 == 1) {
+//        if(nextout2 != pipeoutfds[1]) {
+if(1) {
             // a child process has a own process group
             if(gAppType != kATOptC) { (void)setpgid(pid, pid); }
 
@@ -789,7 +756,8 @@ static BOOL run_external_command(char* program, sObject* nextin, sObject* nextou
     /// the parent process 
     else {
         pid_t pgroup;
-        if(nextout2 == 1) {
+if(1) {
+//        if(nextout2 != pipeoutfds[1]) {
             pgroup = pid; // a child process has a own process group
         }
         else {
@@ -804,10 +772,10 @@ static BOOL run_external_command(char* program, sObject* nextin, sObject* nextou
         /// Therefore xyzsh can't waitpid and deadlock.
         /// Avoid this, xyzsh is ready for a writer process to write buffer to the xyzsh.
         pid_t nextin_reader_pid = -1;
-        if(nextin2 != 0) {
+        if(SFD(nextin).mKind != kFDKindFD && nextin2 != 0) {
             nextin_writer(pgroup, nextin, pipeinfds, pipeoutfds, pipeerrfds, runinfo, program, &nextin_reader_pid);
         }
-        if(nextout2 != 1) {
+        if(SFD(nextout).mKind != kFDKindFD && nextout2 != 1) {
             (void)close(pipeoutfds[1]);
             if(!nextout_reader(nextout, pipeoutfds, runinfo, program)) {
                 // wait everytime
@@ -963,7 +931,9 @@ static BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunI
             break;
 
         case T_STRING: {
-            if(!fd_write(nextout, string_c_str(object))) {
+            char* str = string_c_str(object);
+            int size = strlen(str);
+            if(!fd_write(nextout, str, size)) {
                 sCommand* command = runinfo->mCommand;
                 if(command) {
                     err_msg("signal interrupt9", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -974,7 +944,7 @@ static BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunI
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
-            if(!fd_write(nextout, "\n")) {
+            if(!fd_write(nextout, "\n", 1)) {
                 sCommand* command = runinfo->mCommand;
                 if(command) {
                     err_msg("signal interrupt10", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -1021,7 +991,7 @@ static BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunI
             hash_it* it = hash_loop_begin(object);
             while(it) {
                 char* key = hash_loop_key(it);
-                if(!fd_write(nextout, key)) {
+                if(!fd_write(nextout, key, strlen(key))) {
                     sCommand* command = runinfo->mCommand;
                     if(command) {
                         err_msg("signal interrupt9", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -1032,7 +1002,7 @@ static BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunI
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
-                if(!fd_write(nextout, "\n")) {
+                if(!fd_write(nextout, "\n", 1)) {
                     sCommand* command = runinfo->mCommand;
                     if(command) {
                         err_msg("signal interrupt10", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -1079,12 +1049,11 @@ static BOOL stdin_read_out()
         if(size < 0) {
             return FALSE;
         }
-        buf[size] = 0;
         if(size == 0) {
             break;
         }
         else {
-            if(!fd_write(gStdin, buf)) {
+            if(!fd_write(gStdin, buf, size)) {
                 return FALSE;
             }
         }
@@ -1095,19 +1064,6 @@ static BOOL stdin_read_out()
 
 static BOOL redirect_ready(sObject** nextout, sObject** nextin, int* opened_fd, sObject* pipeout, BOOL last_program, sStatment* statment, sCommand* command, sRunInfo* runinfo)
 {
-    if(last_program) {
-        if(statment->mFlags & STATMENT_GLOBALPIPEOUT) {
-            fd_clear(gGlobalPipe);
-            *nextout = gGlobalPipe;
-        }
-        else {
-            *nextout = pipeout;
-        }
-    }
-    else {
-        *nextout = FD_NEW_STACK(kFDKindBuf, 0);
-    }
-
     int i;
     for(i=0; i<command->mRedirectsNum; i++) {
         int fd;
@@ -1206,7 +1162,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
     }
     else if(statment->mFlags & STATMENT_GLOBALPIPEIN) {
         nextin = FD_NEW_STACK(kFDKindBuf, 0);
-        if(!fd_write(nextin, SFD(gGlobalPipe).fdbuf.mBuf)) {
+        if(!fd_write(nextin, SFD(gGlobalPipe).fdbuf.mBuf, SFD(gGlobalPipe).fdbuf.mBufLen)) {
             sCommand* command = runinfo->mCommand;
             if(command) {
                 err_msg("signal interrupt12", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -1231,6 +1187,25 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
         runinfo->mFilter = i != 0 || (statment->mFlags & (STATMENT_CONTEXTPIPE|STATMENT_GLOBALPIPEIN));
 
         sObject* nextout;
+        if(last_program) {
+            if(statment->mFlags & STATMENT_GLOBALPIPEOUT) {
+                fd_clear(gGlobalPipe);
+                nextout = gGlobalPipe;
+            }
+            else {
+                nextout = pipeout;
+            }
+        }
+        else {
+            nextout = FD_NEW_STACK(kFDKindBuf, 0);
+        }
+
+        /// expand env ///
+        if(!sCommand_expand_env_redirect(command, nextin, nextout, runinfo))
+        {
+            return FALSE;
+        }
+
         int opened_fd = -1;
         if(!redirect_ready(&nextout, &nextin, &opened_fd, pipeout, last_program, statment, command, runinfo)) {
             return TRUE;
